import os
import sys

import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtQml as QtQml
import PyQt5.QtQuick as QtQuick
import PyQt5.QtWidgets as QtWidgets
import sip

qml = "LineOfSightLocation.qml"
win = (sys.platform == "win32")

if win:
    os.environ["PATH"] += ";C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\bin\\release"
    #os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"

    # The following line is strictly for VMWare Fusion compatibility
    os.environ["QT_ANGLE_PLATFORM"] = "d3d9"

import Runtime.Qt
Rt = Runtime.Qt.Esri.ArcGISRuntime

class LineOfSightLocation(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self._bInit = False
        self._app = None
        self._engine = None
        self._qw = None
        self._sc = None
        self._sv = None
        self._los = None
        self._bCalc = False

    def Setup(self):
        if self._bInit:
            return True

        # Load QML and get window
        self._app = QtWidgets.QApplication(sys.argv)
        if win:
            QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_UseOpenGLES)
        engine = QtQml.QQmlApplicationEngine()
        if win:
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\qml")
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\toolkit\\Import")
        else:
            home = os.environ["HOME"]
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/macOS/x64/qml")
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/toolkit/Import")
        self._engine = engine
        engine.load(QtCore.QUrl("../Qt/RuntimeQML/" + qml))
        rootlist = engine.rootObjects()
        if len(rootlist) == 0:
            print("Error loading QML.")
            return False
        qw = rootlist[0]
        self._qw = qw

        # Get SceneView container
        sc = qw.findChild(QtQuick.QQuickItem, "sceneViewContainer")
        self._sc = sc

        # Add SceneView
        sv = Rt.SceneQuickView(sc)
        self._sv = sv

        # Add Scene
        bm = Rt.Basemap.topographic(sc)
        scene = Rt.Scene(bm, sc)
        surface = Rt.Surface(sc)
        qurl = QtCore.QUrl("http://elevation3d.arcgis.com/arcgis/rest/services/WorldElevation3D/Terrain3D/ImageServer")
        es = Rt.ArcGISTiledElevationSource(qurl, sc)
        eslm = surface.elevationSources()
        eslm.append(es)
        scene.setBaseSurface(surface)
        sv.setArcGISScene(scene)

        # Add AnalysisOverlay
        ao = Rt.AnalysisOverlay(sc)
        aolm = sv.analysisOverlays()
        aolm.append(ao)

        # Set initial Viewpoint
        ptVP = Rt.Point(-73.0815, -49.3272, 4059, Rt.SpatialReference.wgs84())
        cam = Rt.Camera(ptVP, 11, 62, 0)
        sv.setViewpointCamera(cam)

        # Create LocationLineOfSight
        ptObs = Rt.Point(-73.06958032962375, -49.253112971555446, 2000, Rt.SpatialReference.wgs84())
        ptTarg = Rt.Point(-73.079266999709162, -49.300457676730559, 1312, Rt.SpatialReference.wgs84())
        los = Rt.LocationLineOfSight(ptObs, ptTarg, sc)
        self._los = los
        alm = ao.analyses()
        alm.append(los)
        Rt.LineOfSight.setVisibleColor(QtGui.QColor("cyan"))
        Rt.LineOfSight.setObstructedColor(QtGui.QColor("magenta"))
        Rt.LineOfSight.setLineWidth(2.0)

        # Connect signals
        sc.widthChanged.connect(self.SizeChanged)
        sc.heightChanged.connect(self.SizeChanged)
        sv.mouseClicked.connect(self.MouseClicked)
        sv.mousePressedAndHeld.connect(self.MousePressedAndHeld)
        sv.mouseMoved.connect(self.MouseMoved)
        sv.mouseReleased.connect(self.MouseReleased)
        self._bInit = True
        return True

    def Show(self):
        if not self._bInit:
            return
        self._qw.show()

    def Exec(self):
        self._app.exec_()

    @QtCore.pyqtSlot()
    def SizeChanged(self):
        self._sv.setWidth(self._sc.width())
        self._sv.setHeight(self._sc.height())
    
    @QtCore.pyqtSlot("QMouseEvent&")
    def MouseClicked(self, arg):
        event = sip.cast(arg, QtGui.QMouseEvent)
        pt = self._sv.screenToBaseSurface(event.x(), event.y())
        self._los.setTargetLocation(pt)
    
    @QtCore.pyqtSlot()
    def MousePressedAndHeld(self):
        self._bCalc = True
    
    @QtCore.pyqtSlot("QMouseEvent&")
    def MouseMoved(self, arg):
        if not self._bCalc:
            return
        event = sip.cast(arg, QtGui.QMouseEvent)
        pt = self._sv.screenToBaseSurface(event.x(), event.y())
        self._los.setTargetLocation(pt)

    @QtCore.pyqtSlot()
    def MouseReleased(self):
        self._bCalc = False

if __name__ == "__main__":
    rt = LineOfSightLocation()
    if rt.Setup():
        rt.Show()
        rt.Exec()
        del rt
    print("--end--")
