import os
import sys

import PyQt5.QtCore as QtCore
import PyQt5.QtQml as QtQml
import PyQt5.QtQuick as QtQuick
import PyQt5.QtWidgets as QtWidgets
import sip

qml = "QuickUI.qml"
win = (sys.platform == "win32")
key = "runtimelite,1000,rud9116709352,none,JFB3LNBHPBK2F5KHT112"

if win:
    os.environ["PATH"] += ";C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\bin\\release"
    #os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"

import Runtime.Example
Rt = Runtime.Example.Esri.ArcGISRuntime

class RuntimeQuick(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self._bInit = False
        self._app = None
        self._engine = None
        self._qw = None
        self._mc = None
        self._lblStatus = None
        self._lblScale = None
        self._btnSignIn = None
        self._btnSkip = None
        self._mv = None
        self._av = None
        self._am = None
        self._portal = None

    def Setup(self):
        if self._bInit:
            return True

        # Load QML and get window
        self._app = QtWidgets.QApplication(sys.argv)
        if win:
            QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_UseOpenGLES)
        engine = QtQml.QQmlApplicationEngine()
        if win:
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\qml")
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\toolkit\\Import")
        else:
            home = os.environ["HOME"]
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/macOS/x64/qml")
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/toolkit/Import")
        self._engine = engine
        engine.load(QtCore.QUrl("../Qt/RuntimeQML/" + qml))
        rootlist = engine.rootObjects()
        if len(rootlist) == 0:
            print("Error loading QML.")
            return False
        qw = rootlist[0]
        self._qw = qw

        # Get UI items
        mc = qw.findChild(QtQuick.QQuickItem, "mapViewContainer")
        self._mc = mc
        self._lblStatus = qw.findChild(QtQuick.QQuickItem, "lblStatus")
        self._lblScale = qw.findChild(QtQuick.QQuickItem, "lblScale")
        self._btnSignIn = qw.findChild(QtQuick.QQuickItem, "btnSignIn")
        self._btnSkip = qw.findChild(QtQuick.QQuickItem, "btnSkip")

        # Add MapView
        bQmlTest = False
        bPropertyTest = True
        mv = Rt.MapQuickView(mc)
        self._mv = mv
        mv.setWrapAroundMode(Rt.WrapAroundMode.Disabled)

        # Add AuthenticationView/AuthenticationManager
        src = b"import Esri.ArcGISRuntime.Toolkit.Dialogs 100.2"
        src += b"\nAuthenticationView {}"
        qc = QtQml.QQmlComponent(engine)
        qc.setData(src, QtCore.QUrl())
        av = qc.create()
        if av is None:
            print("Error creating AuthenticationView:")
            for err in qc.errors():
                print(err.toString())
            return False
        self._av = av
        av.setParentItem(qw.contentItem())
        am = Rt.AuthenticationManager.instance()
        av.setProperty("authenticationManager", am)
        self._am = am
        
        # Create Portal object
        self._portal = Rt.Portal(True)

        # Connect signals
        mc.widthChanged.connect(self.SizeChanged)
        mc.heightChanged.connect(self.SizeChanged)
        self._btnSignIn.clicked.connect(self.SignIn)
        self._btnSkip.clicked.connect(self.Skip)
        mv.mapScaleChanged.connect(self.ScaleChanged)
        mv.setVisible(False)
        am.authenticationChallenge.connect(self.AuthenticationChallenge)
        self._portal.loadStatusChanged.connect(self.LoadStatusChanged)

        self._bInit = True
        return True

    def Show(self):
        if not self._bInit:
            return
        self._qw.show()

    def Exec(self):
        self._app.exec_()

    def Status(self, msg, err):
        self._lblStatus.setProperty("text", msg)
        if err:
            self._lblStatus.setProperty("color", "red")
        else:
            self._lblStatus.setProperty("color", "blue")

    @QtCore.pyqtSlot()
    def SizeChanged(self):
        self._mv.setWidth(self._mc.width())
        self._mv.setHeight(self._mc.height())

    @QtCore.pyqtSlot()
    def SignIn(self):
        self._btnSignIn.setEnabled(False)
        self._portal.load()
        
    @QtCore.pyqtSlot()
    def Skip(self):
        self.SetLicense(key)

    @QtCore.pyqtSlot()
    def ScaleChanged(self):
        d = self._mv.mapScale()
        msg = "Scale: " + format(d, ".0f")
        self._lblScale.setProperty("text", msg)

    @QtCore.pyqtSlot()
    def LoadStatusChanged(self):
        ls = self._portal.loadStatus()
        print("Load Status = " + str(ls))
        if ls == Rt.LoadStatus.Loaded:
            self.PortalLoaded()
        elif ls == Rt.LoadStatus.FailedToLoad:
            self._portal.retryLoad()

    @QtCore.pyqtSlot(Rt.AuthenticationChallenge)
    def AuthenticationChallenge(self, arg):
        ac = sip.cast(arg, Rt.AuthenticationChallenge)
        acType = ac.authenticationChallengeType()
        print("Challenge = " + str(acType))
        self._av.setProperty("visible", "true")

    @QtCore.pyqtSlot()
    def LoadMap(self):
        bm = Rt.Basemap.topographic(self._mv)
        mp = Rt.Map(bm, self._mv)
        self._mv.setMap(mp)
        self._mv.setVisible(True)
        self._btnSkip.setEnabled(False)

    def PortalLoaded(self):
        info = self._portal.portalInfo()
        self.SetLicense(info.licenseInfo())

    def SetLicense(self, InfoKey):
        env = Rt.ArcGISRuntimeEnvironment.instance()
        result = env.setLicense(InfoKey)
        status = result.licenseStatus()
        if status != Rt.LicenseStatus.Valid:
            self.Status("Invalid license", True)
            return False
        if not env.initialize():
            self.Status("Initialization failed", True)
            return False
        lic = env.license()
        ll = lic.licenseLevel()
        msg = "Initialized at license level: "
        if ll == Rt.LicenseLevel.Developer:
            msg += "Developer"
        elif ll == Rt.LicenseLevel.Lite:
            msg += "Lite"
        elif ll == Rt.LicenseLevel.Basic:
            msg += "Basic"
        elif ll == Rt.LicenseLevel.Standard:
            msg += "Standard"
        elif ll == Rt.LicenseLevel.Advanced:
            msg += "Advanced"
        self.Status(msg, False)
        self.LoadMap()

if __name__ == "__main__":
    rt = RuntimeQuick()
    if rt.Setup():
        rt.Show()
        rt.Exec()
        del rt
    print("--end--")
