#include "benchmark.h"

#include <QThread>
#include <QDateTime>
#include <QList>

#include <GeometryEngine.h>
#include <SpatialReference.h>
#include <PointCollection.h>
#include <MultipointBuilder.h>
#include <Multipoint.h>
#include <Polygon.h>
#include <ImmutablePartCollection.h>
#include <ImmutablePart.h>

using namespace Esri::ArcGISRuntime;

Benchmark::Benchmark(QObject *parent) : QObject(parent)
{
    // Seed random number generator
    srand(QDateTime::currentSecsSinceEpoch());
}

Benchmark::~Benchmark()
{
}

void Benchmark::start()
{

    int iNumPolygons = 100000;
    int iRec = 0;
    double dSumArea = 0.0;
    qint64 iSumVertices = 0;

    // Begin benchmark

    emit status(QString("Processing..."), false);
    QDateTime dtStart = QDateTime::currentDateTime();
    for (int i = 0; i < iNumPolygons; i++)
    {

        try
        {

            // Create random polygon

            BenchmarkResult result = this->generatePolygon();
            if (result.numVertices == 0)
                continue; // This should never happen
            dSumArea += result.area;
            iSumVertices += result.numVertices;

        }
        catch (...)
        {
            emit status(QString("Error processing polygon"), true);
            return;
        }

        iRec++;
        if (iRec % 10000 == 0)
            emit status(QString("Processing..." + QString::number(iRec)), false);

    }

    // Completed

    double dArea = 0.0;
    double dNumVertices = 0.0;
    if (iRec > 0)
    {
        dArea = dSumArea / iRec;
        dNumVertices = iSumVertices / iRec;
    }
    QString qsMsg = QString("Avg area = ") + QString::number(dArea, 'f', 0);
    qsMsg += " / Avg pts = " + QString::number(dNumVertices, 'f', 0);
    emit status(qsMsg, false);
    QDateTime dtEnd = QDateTime::currentDateTime();
    qint64 secondsElapsed = dtStart.secsTo(dtEnd);
    emit completed(secondsElapsed);

}

BenchmarkResult Benchmark::generatePolygon()
{

    // Set up random point generation

    int iNumPoints = 100;
    SpatialReference sr = SpatialReference::webMercator();
    double dXMin = -12432342;
    double dYMin = 4182468;
    double dXMax = -12432184;
    double dYMax = 4182624;

    // Generate random MultiPoint

    PointCollection* pc = new PointCollection(sr);
    for (int i = 0; i < iNumPoints; i++)
    {
        double dX = (double)rand() / ((double)RAND_MAX + 1) * (dXMax - dXMin) + dXMin;
        double dY = (double)rand() / ((double)RAND_MAX + 1) * (dYMax - dYMin) + dYMin;
        pc->addPoint(dX, dY);
    }
    MultipointBuilder *mb = new MultipointBuilder(sr);
    mb->setPoints(pc);
    Multipoint mp(mb->toGeometry());

    // Generate convex hull

    BenchmarkResult result;
    result.area = 0;
    result.numVertices = 0;
    Geometry geom = GeometryEngine::convexHull(mp);
    if (geom.isEmpty())
        return result; // This should never happen
    Polygon poly = (Polygon)geom;
    ImmutablePartCollection ipc = poly.parts();
    ImmutablePart ip = ipc.part(0);

    //  Get area and vertex count

    result.area = GeometryEngine::area(poly);
    result.numVertices = ip.pointCount();
    return result;

}
