import QtQuick 2.6
import QtQuick.Controls 1.4
import QtQuick.Controls.Styles 1.4
import QtQuick.Layouts 1.3

import Esri.ArcGISRuntime 100.2
import Esri.ArcGISExtras 1.1
import Esri.ArcGISRuntime.Toolkit.Dialogs 100.2

ApplicationWindow
{
    width: 800
    height: 600
    title: "QuickApp"

    toolBar: ToolBar
    {
        style: ToolBarStyle
        {
            background: Rectangle
            {
                implicitHeight: 16
                implicitWidth: 200
                color: "lightgreen"
                Rectangle
                {
                    anchors.bottom: parent.bottom
                    width: parent.width
                    height: 1
                    color: "green"
                }
            }
        }

        Component
        {
            id: styleToolButton
            ButtonStyle
            {
                background: Rectangle
                {
                    implicitHeight: 30
                    implicitWidth: 50
                    color: "transparent"
                }
                label: Rectangle
                {
                    implicitHeight: 30
                    implicitWidth: 50
                    gradient: Gradient
                    {
                        GradientStop{position: 0; color: control.pressed ? "#222" : "#888"}
                        GradientStop{position: 1; color: control.pressed ? "#888" : "#222"}
                    }
                    radius: 10
                    Label
                    {
                        anchors.centerIn: parent
                        color: control.enabled ? "white" : "black"
                        text: control.text
                        font.bold: true
                    }
                }
            }
        }

        RowLayout
        {
            anchors.centerIn: parent

            ToolButton
            {
                id: btnSignIn
                objectName: "btnSignIn"
                text: "Sign In"
                onClicked: signIn();
                style: styleToolButton
                enabled: true
            }

            ToolButton
            {
                id: btnSkip
                objectName: "btnSkip"
                text: "Skip"
                onClicked: loadMap();
                style: styleToolButton
                enabled: true
            }

        }

    }

    Rectangle
    {
        anchors.fill: parent
        objectName: "mapViewContainer"

        MapView
        {
            id: mapView
            objectName: "mapView"
            anchors.fill: parent
            wrapAroundMode: Enums.WrapAroundModeDisabled
            visible: false
            onMapScaleChanged:
            {
                if (mapView.visible)
                    showScale();
            }
        }

    }

    statusBar: StatusBar
    {
        style: StatusBarStyle
        {
            padding.left: 5
            background: Rectangle
            {
                implicitHeight: 16
                implicitWidth: 200
                gradient: Gradient
                {
                    GradientStop{color: "#fee" ; position: 0}
                    GradientStop{color: "#faa" ; position: 1}
                }
                Rectangle
                {
                    anchors.top: parent.top
                    width: parent.width
                    height: 1
                    color: "red"
                }
            }
        }

        Grid
        {
            anchors.fill: parent
            columns: 2
            spacing: 100

            Label
            {
                id: lblStatus
                width: 100
                objectName: "lblStatus"
                text: "OK"
                color: "blue"
                font.bold: true
            }

            Label
            {
                id: lblScale
                width: 100
                objectName: "lblScale"
                text: "Scale = n/a"
                color: "blue"
                font.bold: true
            }
        }

    }

    AuthenticationView
    {
        id: authView
        authenticationManager: AuthenticationManager
    }

    // Non-UI items

    Portal
    {
        id: portal
        loginRequired: true
        onLoadStatusChanged:
        {
            if (loadStatus === Enums.LoadStatusLoaded)
                portalLoaded();
            else if (loadStatus === Enums.LoadStatusFailedToLoad)
                retryLoad();
        }

    }

    // Functions

    function alertMessage(message, error)
    {
        lblStatus.text = message;
        if (error)
            lblStatus.color = "red";
        else
            lblStatus.color = "blue";
    }

    function showScale()
    {
        var scale = mapView.mapScale;
        var msg = "Scale = 1:" + scale.toFixed(0);
        lblScale.text = msg;
    }

    function signIn()
    {
        btnSignIn.enabled = false;
        portal.load();
    }

    function portalLoaded()
    {

        // Display license level

        var info = portal.portalInfo;
        var licenseInfo = info.licenseInfo;
        var result = ArcGISRuntimeEnvironment.setLicense(licenseInfo);
        if (!result)
        {
            alertMessage("Licensing failed.", true);
            return;
        }
        var message = "License level = ";
        var licenseLevel = ArcGISRuntimeEnvironment.license.licenseLevel;
        if (licenseLevel === Enums.LicenseLevelDeveloper)
            message += "Developer";
        else if (licenseLevel === Enums.LicenseLevelLite)
            message += "Lite";
        else if (licenseLevel === Enums.LicenseLevelBasic)
            message += "Basic";
        else if (licenseLevel === Enums.LicenseLevelStandard)
            message += "Standard";
        else if (licenseLevel === Enums.LicenseLevelAdvanced)
            message += "Advanced";
        alertMessage(message, false);
        if (!mapView.visible)
            loadMap();

    }

    function loadMap()
    {
        var bm = ArcGISRuntimeEnvironment.createObject("BasemapTopographic");
        var map = ArcGISRuntimeEnvironment.createObject("Map", {basemap: bm});
        mapView.map = map;
        mapView.visible = true;
        mapView.focus = true;
        btnSkip.enabled = false;
    }

}
